@@ -2,8 +2,6 @@
  
 """dfsummarizer.dfsummarizer: provides entry point main()."""
  
-__version__ = "0.1.2"
-
 import numpy as np
 import pandas as pd
 import sys
@@ -11,9 +9,11 @@ import os
 
 from .funcs import load_complete_dataframe
 from .funcs import analyse_df
-from .funcs import analyse_df_in_chunks
+from .funcs import analyse_file
+from .funcs import analyse_file_in_chunks
 from .funcs import print_latex
 from .funcs import print_markdown
+from .funcs import print_csv
 from .config import max_filesize
  
 def main():
@@ -28,15 +28,18 @@ def main():
         filesize = os.stat(dataset).st_size
 
         if filesize<max_filesize:
-            df = load_complete_dataframe(dataset)
-            summary = analyse_df(df)
+            # df = load_complete_dataframe(dataset)
+            # summary = analyse_df(df)
+            summary = analyse_file(dataset)
         else:
-            summary = analyse_df_in_chunks(dataset)
+            summary = analyse_file_in_chunks(dataset)
 
         if(format=="latex"):
             print_latex(summary)
         elif(format=="markdown"):
             print_markdown(summary)
+        elif(format=="csv"):
+            print_csv(summary)
         else: 
             print(summary)
 
@@ -44,7 +47,8 @@ def main():
 def print_usage(args):
     """ Command line application usage instrutions. """
     print("USAGE ")
-    print(args[0], " <FORMAT: 'simple' or 'latex' or 'markdown'> <PATH TO DATASET>")
+    print(args[0], " <FORMAT> <PATH TO DATASET>")
+    print("  <FORMAT> - 'simple' or 'csv' or 'latex' or 'markdown'")
     print("  <PATH TO DATASET> - Supported file types: csv, tsv, xls, xlsx, odf")
     print("")
 
