@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-
+from io import StringIO
 import pandas as pd 
 import numpy as np
 import math
@@ -15,6 +15,13 @@ from .FlajoletMartin import FMEstimator
 """
 
 
+########################################################################################
+def analyse_file(path_to_file):
+    df = load_complete_dataframe(path_to_file)
+    summary = analyse_df(df)
+    return summary
+
+
 ########################################################################################
 def analyse_df(df):
     """
@@ -24,7 +31,7 @@ def analyse_df(df):
     colnames = df.columns
     records = len(df)
     df = coerce_dates(df)
-    rez = pd.DataFrame(columns=('Name', 'Type', 'Unique', 'Nulls', 'Min', 'Mean', 'Max'))
+    rez = pd.DataFrame(columns=('Name', 'Type', 'Unique Vals', 'Unique', 'Nulls', 'Min', 'Mean', 'Max'))
 
     for name in colnames:
         nacount = len(df[df[name].isna()])
@@ -61,6 +68,7 @@ def analyse_df(df):
         values_to_add = {
             'Name':name, 
             'Type':valtype,
+            'Unique Vals':unicount, 
             'Unique':unipercent, 
             'Nulls':napercent, 
             'Min':themin, 
@@ -72,7 +80,7 @@ def analyse_df(df):
     return rez
 
 ########################################################################################
-def analyse_df_in_chunks(path_to_file):
+def analyse_file_in_chunks(path_to_file):
     """
         Given a path to a large dataset we will iteratively load it in chunks and build
         out the statistics necessary to summarise the whole dataset.
@@ -91,15 +99,17 @@ def analyse_df_in_chunks(path_to_file):
     summary = generate_final_summary(temp, total_chunks)
     return summary
 
+
 ########################################################################################
 def generate_final_summary(temp, total_chunks):
-    rez = pd.DataFrame(columns=('Name', 'Type', 'Unique', 'Nulls', 'Min', 'Mean', 'Max'))
+    rez = pd.DataFrame(columns=('Name', 'Type', 'Unique Vals', 'Unique', 'Nulls', 'Min', 'Mean', 'Max'))
     for name in temp.keys():
         col = temp[name]
         total = col['nulls'] + col['nonnulls']
         unicount = col['uniques'].estimate()
         if unicount > total:
             uniprop = 1.0
+            unicount = total
         else:
             uniprop = unicount / total
         unipercent = round(100 * uniprop, 1)
@@ -111,6 +121,7 @@ def generate_final_summary(temp, total_chunks):
         values_to_add = {
             'Name':name, 
             'Type': col['type'],
+            'Unique Vals':unicount,
             'Unique':unipercent,
             'Nulls':napercent,
             'Min': col['min'],
@@ -208,6 +219,7 @@ def load_complete_dataframe(path_to_file):
 
     raise ValueError("Unsupported File Type")
 
+
 ########################################################################################
 def infer_type_2( thecolumn, startpoint, unicount, uniques):
     thetype = get_first_non_null_type(thecolumn, startpoint)
@@ -232,6 +244,7 @@ def get_first_non_null_type(thecolumn, startpoint):
         index = index + 1 
     return thetype
 
+
 ########################################################################################
 def infer_type(thetype, unicount, uniques):
      valtype = "Char"
@@ -264,6 +277,7 @@ def infer_type(thetype, unicount, uniques):
                 valtype = "Bool"
      return valtype
 
+
 ########################################################################################
 def count_lines(path_to_file):
     """
@@ -273,6 +287,7 @@ def count_lines(path_to_file):
     for line in open(path_to_file): count += 1
     return count
 
+
 ########################################################################################
 def len_or_null(val):
     """ 
@@ -284,10 +299,12 @@ def len_or_null(val):
     except:
         return np.nan
 
+
 ########################################################################################
 def isNaN(num):
     return num != num
 
+
 ########################################################################################
 def booleanize(x):
     if isNaN(x) :
@@ -304,6 +321,7 @@ def booleanize(x):
     else :
         return 0
 
+
 ########################################################################################
 def coerce_dates(df):
     return df.apply(
@@ -321,12 +339,12 @@ def print_latex(summary):
     print("   \\caption{Data Summary Table}")
     print("   \\label{tab:table1}")
     print("   \\begin{tabular}{l|l|r|r|r|r} ")
-    print("    \\textbf{Name} & \\textbf{Type} & \\textbf{Unique \%} & \\textbf{Nulls \%} & \\textbf{Min} & \\textbf{Mean} & \\textbf{Max}\\\\")
+    print("    \\textbf{Name} & \\textbf{Type} & \\textbf{Unique Vals \%} & \\textbf{Nulls \%} & \\textbf{Min} & \\textbf{Mean} & \\textbf{Max}\\\\")
     print("      \\hline")
     for i in range(len(summary)):
         print("      ", summary.loc[i,"Name"], 
               "&", summary.loc[i,"Type"], 
-              "&", summary.loc[i,"Unique"], "%" 
+              "&", summary.loc[i,"Unique Vals"], "%" 
               "&", summary.loc[i,"Nulls"], "%" 
               "&", summary.loc[i,"Min"], 
               "&", summary.loc[i,"Mean"], 
@@ -335,6 +353,7 @@ def print_latex(summary):
     print("  \\end{center}")
     print("\\end{table}")
 
+
 ########################################################################################
 def get_spaces(spacer):
     rez = ""
@@ -342,6 +361,7 @@ def get_spaces(spacer):
         rez = rez + " "
     return rez
 
+
 ########################################################################################
 def get_type_spacer(t):
     if (t == "Int") :
@@ -354,6 +374,7 @@ def get_type_spacer(t):
         return "  "
     return "   "
 
+
 ########################################################################################
 def get_percent_spacer(p):
     if (p==100.0):
@@ -363,6 +384,7 @@ def get_percent_spacer(p):
     else: 
         return "   "
 
+
 ########################################################################################
 def get_padded_number(n):
     if (n == "-"):
@@ -386,9 +408,13 @@ def get_padded_number(n):
             return get_spaces(3 - after_decimal(n) + adjus) + str(n)+ " "
         if (abs(n)<10000000):
             return get_spaces(2 - after_decimal(n) + adjus) + str(n)+ " "
+        else:
+            number = "{:.2e}".format(n)
+            return get_spaces(2 + adjus) + number + " "
     else:
         return str(n) + " "
 
+
 ########################################################################################
 def after_decimal(n):
     arr = str(n).split(".")
@@ -397,6 +423,12 @@ def after_decimal(n):
     else:
         return -1
 
+########################################################################################
+def print_csv(s):
+    output = StringIO()
+    s.to_csv(output, index=False)
+    print(output.getvalue())
+
 ########################################################################################
 def print_markdown(s):
     longest_name = max(s["Name"].apply(lambda x: len_or_null(x)))
@@ -406,14 +438,14 @@ def print_markdown(s):
         name_spacer = 6
 
     print("| Name ", get_spaces(name_spacer-6), 
-        "| Type   | Unique  | Nulls   |  Min       |  Mean      |  Max       |", sep="")
+        "| Type   | Unique Vals | Nulls   |  Min       |  Mean      |  Max       |", sep="")
     print("| ---- ", get_spaces(name_spacer-6), 
-        "| ------ | ------- | ------- |  ---       |  ----      |  ---       |", sep="")
+        "| ------ | ----------- | ------- |  ---       |  ----      |  ---       |", sep="")
     for i in range(len(s)):
         print("| ", s.loc[i,"Name"], 
             get_spaces(name_spacer - len(s.loc[i,"Name"]) - 1 ), 
             "| ", s.loc[i,"Type"], get_type_spacer(s.loc[i,"Type"]),
-            "| ", get_percent_spacer(s.loc[i,"Unique"]), s.loc[i,"Unique"],"% ", 
+            "|  ", get_padded_number(s.loc[i,"Unique Vals"]), 
             "| ", get_percent_spacer(s.loc[i,"Nulls"]), s.loc[i,"Nulls"],"% ", 
             "| ", get_padded_number(s.loc[i,"Min"]), 
             "| ", get_padded_number(s.loc[i,"Mean"]),
@@ -428,3 +460,4 @@ def round_down(n, decimals=0):
     multiplier = 10 ** decimals
     return math.floor(n * multiplier) / multiplier
 
+
